@@ -33,7 +33,9 @@
 #include <glib-object.h>
 #include <gconf/gconf-client.h>
 #include "mlview-file-selection.h"
-
+#ifdef MLVIEW_WITH_GTK_FILE_CHOOSER
+#include <gtk/gtkfilechooser.h>
+#endif
 
 /**
  *@file
@@ -79,10 +81,10 @@ struct _MlViewAppContextClass {
         GObjectClass parent_class;
 
         /**
-         *Signal emited when the MlViewApp has been initialyzed.
+         *Signal emited when the MlViewApp has been initialized.
          */
         void (*application_initialized) (MlViewAppContext *a_this,
-                                        gpointer a_user_data) ;
+                                         gpointer a_user_data) ;
 
         /**
          *signal emited when an editing
@@ -93,11 +95,19 @@ struct _MlViewAppContextClass {
                                            GtkWidget *a_request_source,
                                            GdkEvent *a_event,
                                            gpointer a_user_data) ;
-
-        void (*view_changed) (MlViewAppContext *a_this,
+        
+        void (*view_swapped) (MlViewAppContext *a_this,
                               gpointer a_old_view,
                               gpointer a_new_view,
                               gpointer a_user_data) ;
+
+        void (*document_name_changed) (MlViewAppContext *a_this,
+                                       gpointer a_doc,
+                                       gpointer a_user_data) ;
+
+        void (*document_undo_state_changed) (MlViewAppContext *a_this,
+                                             gpointer a_mlview_xml_doc,
+                                             gpointer a_user_data) ;
 };
 
 
@@ -113,6 +123,14 @@ enum MlViewTreeEditorsNodeColour {
         NB_OF_TREE_NODE_COLOURS /*this must remain the last entry of the enum*/
 } ;
 
+/* used to change gtk_file_chooser mode (ie open/save) */
+typedef enum {
+	MlViewFileChooserOpenMode = 0,
+	MlViewFileChooserSaveMode
+} MlViewFileChooserMode ;
+
+
+
 /**
  *The public application wide settings
  *data structure.
@@ -134,12 +152,28 @@ struct MlViewAppSettings {
         } network;
 } ;
 
+/**
+ *The structure which contains icons to
+ *represent different types of XML nodes
+ *used across different components of
+ *the application.
+ */
+struct MlViewTypeIcons {
+        GdkPixbuf *element;
+        GdkPixbuf *open_element;
+        GdkPixbuf *text;
+        GdkPixbuf *root;
+        GdkPixbuf *open_root;
+        GdkPixbuf *comment;
+        GdkPixbuf *pi;
+        GdkPixbuf *entity_ref;
+};
 
 /*************************************
  *Public API of the MlViewAppContext
  *************************************/
 
-guint mlview_app_context_get_type (void);
+GType mlview_app_context_get_type (void);
 
 gint *mlview_app_context_get_last_id_ptr (MlViewAppContext *
                                           a_this);
@@ -198,8 +232,11 @@ struct MlViewAppSettings * mlview_app_context_get_settings (MlViewAppContext *a_
 MlViewFileSelection * mlview_app_context_get_file_selector (MlViewAppContext * a_this,
                                                             const gchar * a_title);
 
-void mlview_app_context_set_window_transient_for_app (MlViewAppContext * a_this,
-                                                      GtkWindow * a_window);
+#ifdef MLVIEW_WITH_GTK_FILE_CHOOSER
+GtkFileChooser * mlview_app_context_get_file_chooser (MlViewAppContext *a_this,
+                                                      const gchar *a_title, MlViewFileChooserMode a_mode) ;
+#endif
+
 
 gint mlview_app_context_get_xpm (MlViewAppContext * a_this,
                                  const gchar * a_xpm_name,
@@ -222,7 +259,18 @@ enum MlViewStatus mlview_app_context_notify_view_swapped (MlViewAppContext *a_th
 
 enum MlViewStatus mlview_app_context_notify_application_initialized (MlViewAppContext *a_this) ;
 
+enum MlViewStatus mlview_app_context_notify_document_name_changed (MlViewAppContext *a_this,
+                                                                   gpointer a_document) ;
+
+enum MlViewStatus  mlview_app_context_notify_document_undo_state_changed (MlViewAppContext *a_this,
+                                                                          gpointer a_doc) ;
+
 gboolean mlview_app_context_ask_internal_subset_node_name (gchar **a_name) ;
 
+void mlview_app_context_type_icons_unref (MlViewAppContext *a_this);
+
+struct MlViewTypeIcons *mlview_app_context_type_icons_ref (MlViewAppContext *a_this);
+
+
 G_END_DECLS
 #endif /*__MLVIEW-APP-CONTEXT_H__*/
