@@ -832,10 +832,6 @@ mlview_attribute_picker_new (gchar * a_title,
         mlview_attribute_picker_set_app_context (result,
                                                  a_app_context);
         gtk_window_set_title (GTK_WINDOW (result), a_title);
-        mlview_app_context_set_window_icon (a_app_context,
-                                            GTK_WINDOW (result));
-        gtk_window_set_wmclass (GTK_WINDOW (result),
-                                "attribute-picker", "MlView");
 
         return GTK_WIDGET (result);
 }
