@@ -25,10 +25,12 @@
  */
 
 #include <string.h>
+#include <libxml/hash.h>
 #include <libxml/parser.h>
 #include "mlview-attrs-editor.h"
 #include "mlview-attribute-picker.h"
 #include "mlview-marshal.h"
+#include <gdk/gdkkeysyms.h>
 
 /**
  *@file
@@ -116,8 +118,9 @@ static GtkVBoxClass *gv_parent_class = NULL;
 static guint gv_signals[NUMBER_OF_SIGNALS] = {0};
 
 /*private helper functions*/
-static xmlAttr * mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_list, 
-                                                                        xmlNode * a_node);
+static enum MlViewStatus mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_list, 
+                                                                                xmlNode * a_node,
+                                                                                xmlAttr **a_added_attribute);
 
 /*signal callbacks*/
 
@@ -148,6 +151,11 @@ xml_doc_node_attribute_added_cb (MlViewXMLDocument *a_this,
                                  xmlAttr *a_attr,
                                  MlViewAttrsEditor *a_editor) ;
 
+static gboolean
+tree_key_press_cb (GtkTreeView *a_tree,
+             GdkEventKey *a_event,
+             MlViewAttrsEditor *a_this);
+
 /*
  *Mandatory private functions to comply with the gtk object
  *framework. go to http://www.gtk.org/tutorial/sec-theanatomyofawidget.html
@@ -256,6 +264,11 @@ mlview_attrs_editor_construct (MlViewAttrsEditor *a_this,
                                (PRIVATE (a_this)->model)) ;
         g_return_if_fail (PRIVATE (a_this)->attrs_view) ;
 
+        g_signal_connect (G_OBJECT (PRIVATE (a_this)->attrs_view),
+                         "key_press_event",
+                          G_CALLBACK (tree_key_press_cb),
+                          a_this) ;        
+
         selection = gtk_tree_view_get_selection
                 (PRIVATE (a_this)->attrs_view) ;
         g_return_if_fail (selection) ;
@@ -367,19 +380,29 @@ mlview_attrs_editor_finalize (GObject *a_this)
  *@param a_this the current instance of MlViewAttrsEditor.
  *@param a_node the xmlNode to add the attributes to.
  */
-static xmlAttr *
-mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_this, 
-                                                       xmlNode * a_node) 
+enum MlViewStatus
+mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_this,
+                                                       xmlNode * a_node,
+                                                       xmlAttr **a_added_attribute) 
 {
         gint button = 0;
-        gchar *name_str = NULL, *value_str = NULL;
+        gchar *name_str = NULL, *value_str = NULL, *node_path = NULL ;
         gboolean loop = TRUE;
-        xmlAttr *result = NULL;
+        enum MlViewStatus result = MLVIEW_OK ;
+        xmlAttr *attr = NULL ;
 
-        g_return_val_if_fail (a_this, NULL);
+        g_return_val_if_fail (a_this, MLVIEW_BAD_PARAM_ERROR);
         g_return_val_if_fail (MLVIEW_IS_ATTRS_EDITOR (a_this),
-                              NULL);
+                              MLVIEW_BAD_PARAM_ERROR);
 
+        if (a_node) {
+                mlview_xml_document_get_node_path  (PRIVATE (a_this)->mlview_xml_doc, 
+                                                    a_node, &node_path) ;
+                if (!node_path) {
+                        mlview_utils_trace_info ("Could not get node xpath expr") ;
+                        return MLVIEW_ERROR ;
+                }
+        }
         if (PRIVATE (a_this)->attribute_picker == NULL) {
                 PRIVATE (a_this)->attribute_picker =
                         MLVIEW_ATTRIBUTE_PICKER
@@ -424,7 +447,15 @@ mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_thi
                             (name_str)) {
                                 result = mlview_xml_document_set_attribute 
                                         (PRIVATE (a_this)->mlview_xml_doc,
-                                         a_node, name_str, value_str, TRUE) ;
+                                         node_path, name_str, 
+                                         value_str, TRUE) ;
+
+                                if (result != MLVIEW_OK)
+                                        return MLVIEW_ERROR ;
+                                attr = xmlHasProp (a_node, name_str) ;
+                                if (!attr)
+                                        return MLVIEW_ERROR ;
+
                                 /*
                                  *FIXME: do this only when 
                                  *validation is on 
@@ -433,7 +464,7 @@ mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_thi
                                     && a_node->doc
                                     && a_node->doc->ids) {
                                         xmlID *id = NULL;
-                                        result->atype = 
+                                        attr->atype = 
                                                 XML_ATTRIBUTE_ID;
                                         id = xmlMalloc (sizeof
                                                         (xmlID));
@@ -441,9 +472,9 @@ mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_thi
                                         id->value =
                                                 g_strdup
                                                 (value_str);
-                                        id->attr = result;
+                                        id->attr = attr;
                                         xmlHashAddEntry
-                                                (result->doc->ids,
+                                                (attr->doc->ids,
                                                  (const xmlChar*) 
                                                  value_str, id);
                                 }
@@ -462,6 +493,13 @@ mlview_attrs_editor_add_attribute_to_node_interactive (MlViewAttrsEditor * a_thi
         }
         gtk_widget_hide (GTK_WIDGET
                          (PRIVATE (a_this)->attribute_picker));
+        if (node_path) {
+                g_free (node_path) ;
+                node_path = NULL ;
+        }
+        if (attr) {
+                *a_added_attribute = attr ;
+        }
         return result;
 }
 
@@ -632,7 +670,9 @@ row_selected_cb (GtkTreeSelection *a_tree_sel,
 }
 
 
-
+/*
+   NOT USED ATM
+ *
 static enum MlViewStatus
 get_cur_selected_iter (MlViewAttrsEditor *a_this,
                        GtkTreeIter *a_iter)
@@ -661,6 +701,7 @@ get_cur_selected_iter (MlViewAttrsEditor *a_this,
         }
         return MLVIEW_OK ;
 }
+*/
 
 static void
 xml_doc_node_attribute_name_changed_cb (MlViewXMLDocument *a_this,
@@ -729,13 +770,23 @@ attr_name_cell_edited_cb (GtkCellRendererText *a_renderer,
         GtkTreeIter iter = {0} ;
         guchar *attr_value = NULL, *attr_name = NULL ;
         GtkTreeModel *model = NULL ;
-        xmlAttr *attr = NULL ;
-
+        xmlAttr *attr = NULL, *attr2 = NULL ;
+        const gchar * err_msg = "This node already has an attribute with the same name !" ;
+        gchar *node_path = NULL ;
+        
         g_return_if_fail (a_cell_path && a_attr_name
                           && a_editor && MLVIEW_IS_ATTRS_EDITOR (a_editor)
                           && PRIVATE (a_editor)->current_xml_node
                           && PRIVATE (a_editor)->mlview_xml_doc) ;
         
+        mlview_xml_document_get_node_path (PRIVATE (a_editor)->mlview_xml_doc,
+                                           PRIVATE (a_editor)->current_xml_node,
+                                           &node_path) ;
+        if (!node_path) {
+                mlview_utils_trace_info ("Could not get XPATH expr from XML node") ;
+                return ;
+        }
+
         model = mlview_attrs_editor_get_model (a_editor) ;
         g_return_if_fail (model) ;
         /*get an iter on the currently selected row*/
@@ -754,9 +805,20 @@ attr_name_cell_edited_cb (GtkCellRendererText *a_renderer,
                  */
                 if (a_attr_name
                     && strcmp (a_attr_name, "")) {
+                        if (xmlHasProp (PRIVATE (a_editor)->current_xml_node,
+                                        a_attr_name)) {
+                                /*
+                                 *this node already has an attibute with
+                                 *the same name.
+                                 */
+                                mlview_app_context_error 
+                                        (PRIVATE (a_editor)->app_context,
+                                         _(err_msg)) ;
+                                return ;
+                        }
                         mlview_xml_document_set_attribute
                                 (PRIVATE (a_editor)->mlview_xml_doc,
-                                 PRIVATE (a_editor)->current_xml_node,
+                                 node_path,
                                  a_attr_name, attr_value, TRUE) ;
                 }
         } else {
@@ -764,6 +826,13 @@ attr_name_cell_edited_cb (GtkCellRendererText *a_renderer,
                                     HIDDEN_XML_ATTR_COLUMN, &attr,
                                     -1) ;
                 g_return_if_fail (attr) ;
+                attr2 = xmlHasProp (attr->parent, a_attr_name) ;
+                if (attr2 != attr) {
+                        /*an attribute with the same name already exist !*/
+                        mlview_app_context_error (PRIVATE (a_editor)->app_context,
+                                                  _(err_msg)) ;
+                        return ;
+                }
                 if (a_attr_name
                     && strcmp (a_attr_name, "")
                     && strcmp (a_attr_name, attr_name)) {
@@ -778,6 +847,10 @@ attr_name_cell_edited_cb (GtkCellRendererText *a_renderer,
                                  attr->name, TRUE) ;
                 }
         }
+        if (node_path) {
+                g_free (node_path) ;
+                node_path = NULL ;
+        }
 }
 
 
@@ -792,12 +865,20 @@ attr_value_cell_edited_cb (GtkCellRendererText *a_renderer,
         GtkTreeIter iter = {0} ;
         guchar *attr_value = NULL, *attr_name = NULL ;
         GtkTreeModel *model = NULL ;
+        gchar *node_path = NULL ;
 
         g_return_if_fail (a_cell_path && a_attr_value
                           && a_editor && MLVIEW_IS_ATTRS_EDITOR (a_editor)
                           && PRIVATE (a_editor)->current_xml_node
                           && PRIVATE (a_editor)->mlview_xml_doc) ;
         
+        mlview_xml_document_get_node_path (PRIVATE (a_editor)->mlview_xml_doc,
+                                           PRIVATE (a_editor)->current_xml_node,
+                                           &node_path) ;
+        if (!node_path) {
+                mlview_utils_trace_info ("Could not get XPATH expr from node") ;
+                return ;
+        }
         model = mlview_attrs_editor_get_model (a_editor) ;
         g_return_if_fail (model) ;
         /*get an iter on the currently selected row*/
@@ -814,10 +895,43 @@ attr_value_cell_edited_cb (GtkCellRendererText *a_renderer,
                     && strcmp (a_attr_value, attr_value)) {
                         mlview_xml_document_set_attribute
                                 (PRIVATE (a_editor)->mlview_xml_doc,
-                                 PRIVATE (a_editor)->current_xml_node,
-                                 attr_name, a_attr_value, TRUE) ;
+                                 node_path,
+                                 attr_name, 
+                                 a_attr_value, TRUE) ;
                 }
         }
+        if (node_path) {
+                g_free (node_path) ;
+                node_path = NULL ;
+        }
+}
+
+
+/**
+ *callback for key_press_event in the attributes treeview
+ *remove the selected attribute if DEL key is pressed
+ */
+
+static gboolean
+tree_key_press_cb (GtkTreeView *a_tree,
+             GdkEventKey *a_event,
+             MlViewAttrsEditor *a_this)
+{
+        enum MlViewStatus status = MLVIEW_OK ;
+        GtkTreeIter iter = {0} ;
+        
+        g_return_val_if_fail (a_tree && a_event && a_this, FALSE) ;
+        
+        status = mlview_attrs_editor_get_cur_sel_iter(a_this, &iter);
+        if(status != MLVIEW_OK)
+                return FALSE ;
+
+        if (a_event->keyval == GDK_Delete) {
+                mlview_attrs_editor_remove_attribute (a_this, &iter) ;	  
+                return TRUE ;
+        }
+                      
+        return FALSE ;
 }
 
 /************************
@@ -1508,23 +1622,25 @@ mlview_attrs_editor_clear (MlViewAttrsEditor *a_this)
 enum MlViewStatus
 mlview_attrs_editor_create_attribute (MlViewAttrsEditor *a_this)
 {
-        xmlAttrPtr xml_attr = NULL;
         enum MlViewStatus result = MLVIEW_OK ;
         GtkTreeIter iter = {0} ;
+        xmlAttr *attr = NULL ;
 
         g_return_val_if_fail (a_this != NULL
                               && PRIVATE (a_this)->current_xml_node,
                               MLVIEW_BAD_PARAM_ERROR) ;
-        xml_attr =
+        result =
                 mlview_attrs_editor_add_attribute_to_node_interactive
-                (a_this, PRIVATE (a_this)->current_xml_node);
+                (a_this, PRIVATE (a_this)->current_xml_node,
+                 &attr);
 
-        if (xml_attr == NULL)
-                return MLVIEW_ERROR;
-        result = mlview_attrs_editor_insert_attribute
-                (a_this, &iter, -1, xml_attr) ;
+        if (attr) {
+                result = mlview_attrs_editor_insert_attribute
+                        (a_this, &iter, -1, attr) ;
 
-        return result;
+                return result;
+        }
+        return MLVIEW_ERROR ;
 }
 
 
@@ -1589,7 +1705,10 @@ mlview_attrs_editor_update_attribute (MlViewAttrsEditor *a_this,
 
         status = mlview_attrs_editor_get_row_ref_from_xml_attr 
                 (a_this, a_attr, &row_ref) ;
+
         g_return_val_if_fail (status == MLVIEW_OK, status) ;
+
+        
         if (!row_ref) {
                 status = mlview_attrs_editor_insert_attribute
                         (a_this, &iter, -1 , a_attr) ;
