@@ -60,10 +60,26 @@ struct _MlViewCompletionTablePrivate {
         xmlNode *cur_node;
 
         gboolean dispose_has_run;
+        gboolean foo ;
 };
 
 static GtkTable *gv_parent_class;
 
+static void
+feasible_attribute_selected_cb (GtkTreeSelection *a_tree_selection,
+                                gpointer a_user_data) ;
+static void
+feasible_next_sibling_selected_cb (GtkTreeSelection *a_tree_selection,
+                                   gpointer a_user_data) ;
+
+static void
+feasible_prev_sibling_selected_cb (GtkTreeSelection *a_tree_selection,
+                                   gpointer a_user_data) ;
+
+static void        
+feasible_child_selected_cb (GtkTreeSelection *a_tree_selection,
+                            gpointer a_user_data) ;
+
 static void
 mlview_completion_table_finalize (GObject *a_this)
 {
@@ -115,20 +131,32 @@ mlview_completion_table_class_init (MlViewCompletionTableClass *a_klass)
                 gtk_type_class (GTK_TYPE_TABLE);
 }
 
-static void
-update_list_store (GtkListStore *a_store, GList *a_list)
-{
-        GtkTreeIter iter = {0} ;
 
-        gtk_list_store_clear (a_store);
+
+static void
+update_list_store (MlViewCompletionTable *a_this,
+                   GtkWidget *a_tree_view, 
+                   GList *a_list)
+{
+        GtkTreeModel *store = NULL ;
+        GtkTreeIter iter = {0} ;
+        GtkTreeSelection *selection = NULL ;
+
+        g_return_if_fail (GTK_IS_TREE_VIEW (a_tree_view)) ;
+        
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (a_tree_view)) ;
+        gtk_tree_selection_unselect_all (selection) ;
+        store = gtk_tree_view_get_model (GTK_TREE_VIEW (a_tree_view)) ;
+        gtk_list_store_clear (GTK_LIST_STORE (store));
         while (a_list) {
-                gtk_list_store_append (a_store, &iter);
-                gtk_list_store_set (a_store, &iter, 0, a_list->data, -1);
+                gtk_list_store_append (GTK_LIST_STORE (store), &iter);
+                gtk_list_store_set (GTK_LIST_STORE (store), &iter, 
+                                    0, a_list->data, -1);
                 a_list = g_list_next (a_list);
         }
+        
 }
 
-
 static void
 feasible_attribute_selected_cb (GtkTreeSelection *a_tree_selection,
                                 gpointer a_user_data)
@@ -136,30 +164,55 @@ feasible_attribute_selected_cb (GtkTreeSelection *a_tree_selection,
         MlViewCompletionTable *widget;
         GtkTreeIter iter;
         gchar *str = NULL;
-        xmlAttr *attr;
+        GList * selected_rows = NULL ;
+        GtkTreeModel *model = NULL ;
+        gchar *node_path = NULL ;
+        enum MlViewStatus status = MLVIEW_OK ;
 
         g_return_if_fail (a_tree_selection && GTK_IS_TREE_SELECTION (a_tree_selection));
         g_return_if_fail (a_user_data && MLVIEW_IS_COMPLETION_TABLE (a_user_data));
 
         widget = MLVIEW_COMPLETION_TABLE (a_user_data);
-        
+
         g_return_if_fail (PRIVATE (widget));
         g_return_if_fail (PRIVATE (widget)->cur_node);
 
-        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_children);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_prev_siblings);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_next_siblings);
-
-                gtk_tree_model_get 
+        model = GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_attributes) ;
+        g_return_if_fail (model) ;
+        selected_rows = gtk_tree_selection_get_selected_rows (a_tree_selection,
+                                                              &model) ;
+        if (!selected_rows)
+                return ;
+        
+        mlview_xml_document_get_node_path (PRIVATE (widget)->xml_doc,
+                                           PRIVATE (widget)->cur_node,
+                                           &node_path) ;
+        if (!node_path) {
+                mlview_utils_trace_info ("Could not get XPATH expr from node") ;
+                return ;
+        }
+        if (gtk_tree_model_get_iter (model, &iter, selected_rows->data)) {
+                gtk_tree_model_get
                         (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_attributes), 
                          &iter, 0, &str, -1);
-                
-                attr = mlview_xml_document_set_attribute (PRIVATE (widget)->xml_doc,
-                                                          PRIVATE (widget)->cur_node,
-                                                          str, "value", TRUE);
 
-                g_return_if_fail (attr);
+                status = mlview_xml_document_set_attribute (PRIVATE (widget)->xml_doc,
+                                                            node_path,
+                                                            str, 
+                                                            "value", 
+                                                            TRUE);
+                if (status != MLVIEW_OK) {
+                        mlview_utils_trace_info ("Could not set attribute") ;                        
+                }
+        }
+        /*cleanup:*/
+        if (selected_rows) {
+                g_list_foreach (selected_rows, (GFunc)gtk_tree_path_free, NULL) ;
+                g_list_free (selected_rows) ;
+        }
+        if (node_path) {
+                g_free (node_path) ;
+                node_path = NULL ;
         }
 }
 
@@ -167,40 +220,77 @@ static void
 feasible_next_sibling_selected_cb (GtkTreeSelection *a_tree_selection,
                                    gpointer a_user_data)
 {
+        enum MlViewStatus status = MLVIEW_OK ;
         MlViewCompletionTable *widget;
         GtkTreeIter iter;
         gchar *str = NULL;
-        xmlNode *new_node = NULL,
-                *added_node = NULL;
+        xmlNode *new_node = NULL ;
+        GList *selected_rows = NULL ;
+        GtkTreeModel *model = NULL ;
+        gchar *node_path = NULL ;
 
-        g_return_if_fail (a_tree_selection && GTK_IS_TREE_SELECTION (a_tree_selection));
-        g_return_if_fail (a_user_data && MLVIEW_IS_COMPLETION_TABLE (a_user_data));
-
-        widget = MLVIEW_COMPLETION_TABLE (a_user_data);
+        g_return_if_fail (a_tree_selection 
+                          && GTK_IS_TREE_SELECTION (a_tree_selection));
+        g_return_if_fail (a_user_data 
+                          && MLVIEW_IS_COMPLETION_TABLE (a_user_data));
         
-        g_return_if_fail (PRIVATE (widget));
+        widget = MLVIEW_COMPLETION_TABLE (a_user_data);
+        g_return_if_fail (widget && PRIVATE (widget)) ;
+        PRIVATE (widget)->foo = TRUE ;        
         g_return_if_fail (PRIVATE (widget)->cur_node);
 
-        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_children);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_prev_siblings);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_attributes);
+        mlview_xml_document_get_node_path (PRIVATE (widget)->xml_doc,
+                                           PRIVATE (widget)->cur_node,
+                                           &node_path) ;
+        if (!node_path) {
+                mlview_utils_trace_info ("Could not get XPATH expr from node") ;
+                return ;
+        }
+        model = GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_next_siblings) ;
+        g_return_if_fail (model) ;
 
-                gtk_tree_model_get 
-                        (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_next_siblings), 
+        selected_rows = gtk_tree_selection_get_selected_rows (a_tree_selection,
+                                                              &model) ;
+        if (!selected_rows)
+                return ;
+
+        if (gtk_tree_model_get_iter (model, &iter,
+                                     selected_rows->data)) {
+                gtk_tree_selection_unselect_iter (a_tree_selection, &iter) ;
+
+                gtk_tree_model_get
+                        (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_next_siblings),
                          &iter, 0, &str, -1);
-                
+
                 if (strcmp (str, "#PCDATA") == 0) {
                         new_node = xmlNewNode (NULL, "text");
                         new_node->type = XML_TEXT_NODE;
                 } else 
                         new_node = xmlNewNode (NULL, str);
-                
-                added_node = mlview_xml_document_insert_next_sibling_node
-                        (PRIVATE (widget)->xml_doc, PRIVATE (widget)->cur_node,
-                         new_node, TRUE, TRUE);
-                
-                g_return_if_fail (added_node);
+#ifndef MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION
+                status = mlview_xml_document_insert_next_sibling_node
+                        (PRIVATE (widget)->xml_doc, 
+                         node_path,
+                         new_node, 
+                         TRUE, TRUE);
+#else
+                status = mlview_xml_document_insert_next_sibling_node
+                        (PRIVATE (widget)->xml_doc, 
+                         node_path,
+                         new_node, FALSE, TRUE);
+#endif
+                g_return_if_fail (status == MLVIEW_OK);
+        }
+        gtk_tree_selection_unselect_all (a_tree_selection) ;
+
+        /*cleanup*/
+        if (selected_rows) {
+                g_list_foreach (selected_rows, (GFunc)gtk_tree_path_free, NULL) ;
+                g_list_free (selected_rows) ;
+        }
+        if (node_path) {
+                g_free (node_path) ;
+                node_path = NULL ;
         }
 }
 
@@ -208,53 +298,83 @@ static void
 feasible_prev_sibling_selected_cb (GtkTreeSelection *a_tree_selection,
                                    gpointer a_user_data)
 {
+        enum MlViewStatus status = MLVIEW_OK ;
         MlViewCompletionTable *widget;
         GtkTreeIter iter;
         gchar *str = NULL;
-        xmlNode *new_node = NULL,
-                *added_node = NULL;
+        xmlNode *new_node = NULL ;
+        GList *selected_rows = NULL ;
+        GtkTreeModel *model = NULL ;
+        gchar *node_path = NULL ;
 
-        g_return_if_fail (a_tree_selection && GTK_IS_TREE_SELECTION (a_tree_selection));
-        g_return_if_fail (a_user_data && MLVIEW_IS_COMPLETION_TABLE (a_user_data));
+        g_return_if_fail (a_tree_selection
+                          && GTK_IS_TREE_SELECTION (a_tree_selection));
+        g_return_if_fail (a_user_data 
+                          && MLVIEW_IS_COMPLETION_TABLE (a_user_data));
 
         widget = MLVIEW_COMPLETION_TABLE (a_user_data);
-        
+        g_return_if_fail (widget) ;
         g_return_if_fail (PRIVATE (widget));
         g_return_if_fail (PRIVATE (widget)->cur_node);
 
-        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_children);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_next_siblings);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_attributes);
+        model = GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_prev_siblings) ;
+        g_return_if_fail (model) ;
 
-                gtk_tree_model_get 
-                        (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_prev_siblings), 
+        mlview_xml_document_get_node_path (PRIVATE (widget)->xml_doc,
+                                           PRIVATE (widget)->cur_node,
+                                           &node_path) ;
+        if (!node_path) {
+                mlview_utils_trace_info ("could not get XPATH expr from node") ; 
+                return ;
+        }
+        selected_rows = gtk_tree_selection_get_selected_rows (a_tree_selection,
+                                                              &model) ;
+        if (!selected_rows)
+                return ;
+
+        if (gtk_tree_model_get_iter (model, &iter,
+                                     selected_rows->data)) {
+                gtk_tree_model_get
+                        (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_prev_siblings),
                          &iter, 0, &str, -1);
-                
+
                 if (strcmp (str, "#PCDATA") == 0) {
                         new_node = xmlNewNode (NULL, "text");
                         new_node->type = XML_TEXT_NODE;
                 } else 
                         new_node = xmlNewNode (NULL, str);
-                
-                added_node = mlview_xml_document_insert_prev_sibling_node
-                        (PRIVATE (widget)->xml_doc, PRIVATE (widget)->cur_node,
-                         new_node, TRUE, TRUE);
-                
-                g_return_if_fail (added_node);
+#ifndef MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION                
+                status = mlview_xml_document_insert_prev_sibling_node
+                        (PRIVATE (widget)->xml_doc, 
+                         node_path, new_node, 
+                         TRUE, TRUE);
+#else
+                status = mlview_xml_document_insert_prev_sibling_node
+                        (PRIVATE (widget)->xml_doc,
+                         node_path, new_node,
+                         FALSE, TRUE);
+#endif
+                g_return_if_fail (status == MLVIEW_OK);
+        }
+        /*cleanup*/
+        if (selected_rows) {
+                g_list_foreach (selected_rows, (GFunc)gtk_tree_path_free, NULL) ;
+                g_list_free (selected_rows) ;
         }
 }
 
-static void        
+static void
 feasible_child_selected_cb (GtkTreeSelection *a_tree_selection,
                             gpointer a_user_data)
 {
         MlViewCompletionTable *widget;
         GtkTreeIter iter;
-        gchar *str = NULL;
-        xmlNode *new_node = NULL,
-                *added_node = NULL;
-
+        gchar *str = NULL, *cur_node_path = NULL;
+        xmlNode *new_node = NULL ;
+        enum MlViewStatus status = MLVIEW_OK ;
+        GList *selected_rows = NULL ;
+        GtkTreeModel *model = NULL ;
+        
         g_return_if_fail (a_tree_selection && GTK_IS_TREE_SELECTION (a_tree_selection));
         g_return_if_fail (a_user_data && MLVIEW_IS_COMPLETION_TABLE (a_user_data));
 
@@ -263,26 +383,60 @@ feasible_child_selected_cb (GtkTreeSelection *a_tree_selection,
         g_return_if_fail (PRIVATE (widget));
         g_return_if_fail (PRIVATE (widget)->cur_node);
 
-        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_prev_siblings);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_next_siblings);
-                gtk_tree_selection_unselect_all (PRIVATE (widget)->selections.feasible_attributes);
+        model = GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_children) ;
+        g_return_if_fail (model) ;
+        
+        selected_rows = gtk_tree_selection_get_selected_rows (a_tree_selection,
+                                                              &model) ;
+        if (!selected_rows)
+                return ;
 
-                gtk_tree_model_get 
-                        (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_children), 
+        if (gtk_tree_model_get_iter (model, &iter, selected_rows->data)) {
+                gtk_tree_model_get
+                        (GTK_TREE_MODEL (PRIVATE (widget)->models.feasible_children),
                          &iter, 0, &str, -1);
-
+                
                 if (strcmp (str, "#PCDATA") == 0) {
                         new_node = xmlNewNode (NULL, "text");
                         new_node->type = XML_TEXT_NODE;
                 } else 
                         new_node = xmlNewNode (NULL, str);
+
+                mlview_xml_document_get_node_path (PRIVATE (widget)->xml_doc,
+                                                   PRIVATE (widget)->cur_node,
+                                                   &cur_node_path) ;
+                if (!cur_node_path) {
+                        mlview_utils_trace_info ("Faild to get node path") ;
+                        goto cleanup ;
+                }
                 
-                added_node = mlview_xml_document_add_child_node
-                        (PRIVATE (widget)->xml_doc, PRIVATE (widget)->cur_node,
+#ifndef MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION
+               status = mlview_xml_document_add_child_node
+                        (PRIVATE (widget)->xml_doc, 
+                         cur_node_path,
                          new_node, TRUE, TRUE);
-                
-                g_return_if_fail (added_node);
+#else
+               status = mlview_xml_document_add_child_node
+                        (PRIVATE (widget)->xml_doc, 
+                         cur_node_path,
+                         new_node, FALSE, TRUE);
+#endif
+               if (cur_node_path) {
+                       g_free (cur_node_path) ;
+                       cur_node_path = NULL ;
+               }
+               g_return_if_fail (status == MLVIEW_OK);
+        }
+        new_node = NULL ;
+ cleanup:
+        if (selected_rows) {
+                g_list_foreach (selected_rows, 
+                                (GFunc)gtk_tree_path_free, NULL) ;
+                g_list_free (selected_rows) ;
+        }
+        if (new_node) {
+                xmlFreeNode (new_node) ;
+                new_node = NULL ;
         }
 }
 
@@ -295,31 +449,37 @@ mlview_completion_table_init (MlViewCompletionTable *a_this)
 
         gtk_table_set_col_spacings (GTK_TABLE (a_this), 3);
         gtk_table_set_row_spacings (GTK_TABLE (a_this), 3);
-        
+
         if (PRIVATE (a_this) == NULL) {
                 PRIVATE (a_this) = g_try_malloc (sizeof (MlViewCompletionTablePrivate));
                 g_return_if_fail (PRIVATE (a_this));
         }
-
         memset (PRIVATE (a_this), 0, sizeof (MlViewCompletionTablePrivate));
-        
-        PRIVATE (a_this)->models.feasible_children = 
+
+        PRIVATE (a_this)->models.feasible_children =
                 gtk_list_store_new (1, G_TYPE_STRING);
-        PRIVATE (a_this)->widgets.feasible_children = 
-                gtk_tree_view_new_with_model 
+        PRIVATE (a_this)->widgets.feasible_children =
+                gtk_tree_view_new_with_model
                 (GTK_TREE_MODEL (PRIVATE (a_this)->models.feasible_children));
         renderer = gtk_cell_renderer_text_new ();
-        gtk_tree_view_insert_column_with_attributes 
+        gtk_tree_view_insert_column_with_attributes
                 (GTK_TREE_VIEW
                  (PRIVATE (a_this)->widgets.feasible_children), 1,
                  _("Possible children"), renderer, "text", 0, NULL);
-        selection = gtk_tree_view_get_selection 
-                (GTK_TREE_VIEW 
+        selection = gtk_tree_view_get_selection
+                (GTK_TREE_VIEW
                  (PRIVATE (a_this)->widgets.feasible_children));
+        /*
+         *set the selection to GTK_SELECTION_MULTIPLE to workaround
+         *a gtktreeview re-entrency bug.
+         */
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE) ;
         g_signal_connect (G_OBJECT (selection), "changed",
                           G_CALLBACK (feasible_child_selected_cb),
                           a_this);
         PRIVATE (a_this)->selections.feasible_children = selection;
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE) ;
+
 
         PRIVATE (a_this)->models.feasible_prev_siblings = 
                 gtk_list_store_new (1, G_TYPE_STRING);
@@ -329,34 +489,39 @@ mlview_completion_table_init (MlViewCompletionTable *a_this)
         renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_insert_column_with_attributes 
                 (GTK_TREE_VIEW 
-                 (PRIVATE (a_this)->widgets.feasible_prev_siblings), 
+                 (PRIVATE (a_this)->widgets.feasible_prev_siblings),
                  1,
                  _("Possible previous siblings"), 
                  renderer, "text", 0, 
                  NULL);
-        selection = gtk_tree_view_get_selection 
+        selection = gtk_tree_view_get_selection
                 (GTK_TREE_VIEW 
                  (PRIVATE (a_this)->widgets.feasible_prev_siblings));
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE) ;
         g_signal_connect 
                 (G_OBJECT (selection), "changed",
                  G_CALLBACK (feasible_prev_sibling_selected_cb),
                  a_this);
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE) ;
         PRIVATE (a_this)->selections.feasible_prev_siblings = selection;
-        
+
+
         PRIVATE (a_this)->models.feasible_next_siblings = gtk_list_store_new (1, G_TYPE_STRING);
         PRIVATE (a_this)->widgets.feasible_next_siblings = gtk_tree_view_new_with_model 
                 (GTK_TREE_MODEL (PRIVATE (a_this)->models.feasible_next_siblings));
         renderer = gtk_cell_renderer_text_new ();
-        gtk_tree_view_insert_column_with_attributes 
+        gtk_tree_view_insert_column_with_attributes
                 (GTK_TREE_VIEW 
                  (PRIVATE (a_this)->widgets.feasible_next_siblings), 
                  1,
                  _("Possible next siblings"), 
                  renderer, 
                  "text", 0, NULL);
-        selection = gtk_tree_view_get_selection 
+        selection = gtk_tree_view_get_selection
                 (GTK_TREE_VIEW 
                  (PRIVATE (a_this)->widgets.feasible_next_siblings));
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE) ;
+
         g_signal_connect (G_OBJECT (selection), "changed",
                           G_CALLBACK (feasible_next_sibling_selected_cb),
                           a_this);
@@ -375,9 +540,11 @@ mlview_completion_table_init (MlViewCompletionTable *a_this)
                  _("Possible attributes"), renderer, "text", 0, NULL);
         selection = gtk_tree_view_get_selection 
                 (GTK_TREE_VIEW (PRIVATE (a_this)->widgets.feasible_attributes));
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE) ;
         g_signal_connect (G_OBJECT (selection), "changed",
                           G_CALLBACK (feasible_attribute_selected_cb),
                           a_this);
+
         PRIVATE (a_this)->selections.feasible_attributes = selection;
 
         gtk_table_resize (GTK_TABLE (a_this), 2, 2);
@@ -471,35 +638,49 @@ mlview_completion_table_select_node (MlViewCompletionTable *a_widget,
         app_context = mlview_xml_document_get_app_context 
                 (PRIVATE (a_widget)->xml_doc);
 
+#ifndef MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION
         if (a_node_found->type == XML_ELEMENT_NODE &&
             mlview_xml_document_is_node_valid 
             (PRIVATE (a_widget)->xml_doc,
-             a_node_found)) { 
+             a_node_found))
+#else
+        if (a_node_found->type == XML_ELEMENT_NODE)
+            
+#endif
+        {                
                 mlview_parsing_utils_build_element_name_completion_list
                         (app_context,
                          ADD_CHILD,
                          a_node_found, &list);   
-                update_list_store (PRIVATE (a_widget)->models.feasible_children, list);
+                update_list_store (a_widget, 
+                                   PRIVATE (a_widget)->widgets.feasible_children, 
+                                   list);
                 g_list_free (list);
                 list = NULL;
                 mlview_parsing_utils_build_element_name_completion_list
                         (app_context,
                          INSERT_BEFORE,
                          a_node_found, &list);   
-                update_list_store (PRIVATE (a_widget)->models.feasible_prev_siblings, list);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_prev_siblings, 
+                                   list);
                 g_list_free (list);
                 list = NULL;
                 mlview_parsing_utils_build_element_name_completion_list
                         (app_context,
                          INSERT_AFTER,
                          a_node_found, &list);   
-                update_list_store (PRIVATE (a_widget)->models.feasible_next_siblings, list);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_next_siblings, 
+                                   list);
                 g_list_free (list);
                 list = NULL;
                 mlview_parsing_utils_build_attribute_name_completion_list
                         (app_context, a_node_found,
                          &list, FALSE);
-                update_list_store (PRIVATE (a_widget)->models.feasible_attributes, list);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_attributes, 
+                                   list);
                 g_list_free (list);
                 list = NULL;
 
@@ -508,13 +689,20 @@ mlview_completion_table_select_node (MlViewCompletionTable *a_widget,
                 gtk_widget_set_sensitive (GTK_WIDGET (a_widget), TRUE);
         }
         else {
-                update_list_store (PRIVATE (a_widget)->models.feasible_children, NULL);
-                update_list_store (PRIVATE (a_widget)->models.feasible_prev_siblings, NULL);
-                update_list_store (PRIVATE (a_widget)->models.feasible_next_siblings, NULL);
-                update_list_store (PRIVATE (a_widget)->models.feasible_attributes, NULL);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_children, 
+                                   NULL);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_prev_siblings, 
+                                   NULL);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_next_siblings, 
+                                   NULL);
+                update_list_store (a_widget,
+                                   PRIVATE (a_widget)->widgets.feasible_attributes, 
+                                   NULL);
 
                 PRIVATE (a_widget)->cur_node = NULL;
-
                 gtk_widget_set_sensitive (GTK_WIDGET (a_widget), FALSE);
         }
 }
