@@ -56,41 +56,24 @@ MlViewFileDescriptor *mlview_file_descriptor_new (const gchar *a_file_path);
 
 void mlview_file_descriptor_destroy (MlViewFileDescriptor * a_file_desc);
 
+gchar *mlview_file_descriptor_get_uri (const MlViewFileDescriptor *a_file_desc);
+
 gchar *mlview_file_descriptor_get_file_path (const MlViewFileDescriptor *a_file_desc);
 
-enum MLVIEW_FILE_DESCRIPTOR_STATUS mlview_file_descriptor_fetch_file (MlViewFileDescriptor * a_file_desc,
-                                                                      const gchar * a_file_path);
 void mlview_file_descriptor_set_file_path (MlViewFileDescriptor * a_file_desc,
                                            gchar * file_path);
 
-guint mlview_file_descriptor_get_lmd (const MlViewFileDescriptor *a_file_desc);
-
-void mlview_file_descriptor_set_lmd (MlViewFileDescriptor * a_file_desc,
-                                     guint a_last_modified_time);
-
 void mlview_file_descriptor_update_modified_time (MlViewFileDescriptor * a_file_desc);
 
 gint mlview_file_descriptor_is_modified (const MlViewFileDescriptor * a_file_desc,
                                          gboolean * a_is_modified);
 
-gint mlview_file_descriptor_is_readable (const MlViewFileDescriptor * a_file_desc,
-                                         gboolean * a_is_readable);
-
-gint mlview_file_descriptor_is_writeable (const MlViewFileDescriptor * a_file_desc,
-                                          gboolean * a_is_writeable);
-
-gint mlview_file_descriptor_is_dir (const MlViewFileDescriptor * a_file_desc, 
-                                    gboolean * a_is_dir);
-
 gint mlview_file_descriptor_is_regular_file (const MlViewFileDescriptor * a_file_desc,
                                              gboolean * a_is_reg);
 
-gint mlview_file_descriptor_create_directory (MlViewFileDescriptor * a_file_desc,
-                                              mode_t a_mode);
-
-gint mlview_file_descriptor_create_file (MlViewFileDescriptor * a_file_desc, 
-                                         mode_t a_mode);
-
 gint mlview_file_descriptor_is_local (MlViewFileDescriptor * a_file_desc,
                                       gboolean * a_is_local);
+
+gchar * mlview_file_descriptor_get_mime_type (MlViewFileDescriptor * a_file_desc);
+
 #endif
