@@ -32,8 +32,11 @@
  *The definition of the #MlViewIView interface.
  */
 enum MlVIXMLDocViewSignals {
-	NAME_CHANGED,
-	NB_SIGNALS
+        IS_SWAPPED_OUT,
+	IS_SWAPPED_IN,
+        NAME_CHANGED,
+        APPLICATION_MENU_POPULATING_REQUESTED,
+        NB_SIGNALS
 } ;
 
 static guint gv_signals[NB_SIGNALS] = {0} ;
@@ -57,6 +60,37 @@ mlview_iview_base_init (gpointer a_klass)
 			      NULL, NULL,
 			      mlview_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0, NULL) ;
+
+        gv_signals[IS_SWAPPED_OUT] =
+		g_signal_new ("is-swapped-out",
+			      MLVIEW_TYPE_IVIEW,
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (MlViewIView,
+					       is_swapped_out),
+			      NULL, NULL,
+			      mlview_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0, NULL) ;
+
+        gv_signals[IS_SWAPPED_IN] =
+		g_signal_new ("is-swapped-in",
+			      MLVIEW_TYPE_IVIEW,
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (MlViewIView,
+					       is_swapped_in),
+			      NULL, NULL,
+			      mlview_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0, NULL) ;
+
+        gv_signals[APPLICATION_MENU_POPULATING_REQUESTED] =
+		g_signal_new ("application-menu-populating-requested",
+			      MLVIEW_TYPE_IVIEW,
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (MlViewIView,
+					       application_menu_populating_requested),
+			      NULL, NULL,
+			      mlview_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0, NULL) ;
+
 	initialized = TRUE ;
 }
 
@@ -357,3 +391,37 @@ mlview_iview_get_impl (MlViewIView *a_this, GtkWidget **a_impl)
 	}
 	return MLVIEW_OK ;
 }
+
+enum MlViewStatus
+mlview_iview_notify_swapped_out (MlViewIView *a_this)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_IVIEW (a_this),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        g_signal_emit (G_OBJECT (a_this),
+                       gv_signals[IS_SWAPPED_OUT], 0) ;
+        return MLVIEW_OK ;
+}
+
+
+enum MlViewStatus
+mlview_iview_notify_swapped_in (MlViewIView *a_this)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_IVIEW (a_this),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        g_signal_emit (G_OBJECT (a_this),
+                       gv_signals[IS_SWAPPED_IN], 0) ;
+        return MLVIEW_OK ;
+}
+
+enum MlViewStatus 
+mlview_iview_request_application_menu_populating (MlViewIView *a_this)
+{
+        g_return_val_if_fail (a_this && MLVIEW_IS_IVIEW (a_this),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        g_signal_emit (G_OBJECT (a_this),
+                       gv_signals[APPLICATION_MENU_POPULATING_REQUESTED], 0) ;
+        return MLVIEW_OK ;
+}
