@@ -75,27 +75,50 @@ G_BEGIN_DECLS
 #define MLVIEW_IS_NODE_EDITOR(object) (GTK_CHECK_TYPE((object), MLVIEW_TYPE_NODE_EDITOR))
 #define MLVIEW_IS_NODE_EDITOR_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), MLVIEW_TYPE_NODE_EDITOR))
 
-typedef struct _MlViewNodeEditorPrivate
-MlViewNodeEditorPrivate;
+typedef struct _MlViewNodeEditorPrivate MlViewNodeEditorPrivate;
 typedef struct _MlViewNodeEditor MlViewNodeEditor;
-typedef struct _MlViewNodeEditorClass
-MlViewNodeEditorClass;
+typedef struct _MlViewNodeEditorClass MlViewNodeEditorClass;
+
+/***********************************************
+ *The structure representing the MlViewNodeEditor 
+ *************************************************/
 
-/*======================================================================
- *The structure representing the MlViewNodeEditor object which inherits a GtkHPaned.
- *=====================================================================*/
 struct _MlViewNodeEditor {
-	/*inherits a GtkVPaned */
+	/*inherits a GtkHPaned */
 	GtkHPaned hpaned; 
 	MlViewNodeEditorPrivate *priv ;
 };
 
-/*=======================================================================
- *The exported methods of the MlViewELementEditor object.
- *=======================================================================*/
+struct _MlViewNodeEditorClass {
+	GtkHPanedClass parent_class;
+
+	/*signal default handlers */
+
+	void (*element_changed) (MlViewNodeEditor *a_this,
+				 gpointer a_data);
+
+	void (*element_name_changed) (MlViewNodeEditor * a_this,
+				      xmlNodePtr a_node);
+
+	void (*element_attribute_changed) (MlViewNodeEditor * a_this,
+					   xmlNodePtr a_node);
+
+	void (*element_content_changed) (MlViewNodeEditor * a_this,
+					 xmlNodePtr a_node);
+
+	void (*edit_state_changed) (MlViewNodeEditor * a_this);
+	
+	void (*ungrab_focus_requested) (MlViewNodeEditor *a_this) ;
+
+};
+
+/*****************************************************
+ *The exported methods of the MlViewNodeEditor object.
+ *****************************************************/
+
 guint mlview_node_editor_get_type (void);
 
-GtkWidget *mlview_node_editor_new (MlViewAppContext *a_app_context,
+GtkWidget *mlview_node_editor_new (MlViewAppContext *a_app_context, 
 				   MlViewXMLDocument *a_doc);
 
 void
@@ -129,21 +152,11 @@ enum MlViewStatus mlview_node_editor_connect_to_doc (MlViewNodeEditor *a_this,
 
 enum MlViewStatus mlview_node_editor_disconnect_from_doc (MlViewNodeEditor *a_this,
 							  MlViewXMLDocument *a_doc) ;
-struct _MlViewNodeEditorClass {
-	GtkHPanedClass parent_class;
-	/*signal default handlers */
-	void (*element_changed) (MlViewNodeEditor *
-				 a_editor,
-				 gpointer a_data);
-	void (*element_name_changed) (MlViewNodeEditor *
-				      a_editor,
-				      xmlNodePtr a_node);
-	void (*element_attribute_changed) (MlViewNodeEditor * a_editor,
-					   xmlNodePtr a_node);
-	void (*element_content_changed) (MlViewNodeEditor * a_editor,
-					 xmlNodePtr a_node);
-	void (*edit_state_changed) (MlViewNodeEditor * a_editor);
-};
+
+enum MlViewStatus mlview_node_editor_grab_focus (MlViewNodeEditor *a_this) ;
+
+
+enum MlViewStatus mlview_node_editor_request_ungrab_focus (MlViewNodeEditor *a_this) ;
 
 G_END_DECLS
 #endif /*__MLVIEW_NODE_EDITOR_H__*/
