@@ -32,14 +32,15 @@
 #include <string.h>
 #include <libxml/parser.h>
 #include <libxml/catalog.h>
+#include <libxml/valid.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/SAX.h>
-#include <libxml/xmlerror.h>
 #include <libxml/uri.h>
-#include <libxml/hash.h>
+/*#include <libxml/hash.h>*/
 #include <glade/glade.h>
 #include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <zlib.h>
 
 #include "mlview-utils.h"
@@ -210,18 +211,16 @@ mlview_ext_subs_def_destroy (MlViewExtSubsDef * a_def)
  *to report  MLVIEW_ERRORS 
  */
 static enum MlViewStatus
-load_xml_document_from_local_file (gchar * a_xml_file_name,
+load_xml_document_from_local_file (gchar * a_xml_file_uri,
                                    xmlParserCtxt ** a_parser_context,
                                    gboolean a_store_external_subset_info,
                                    MlViewAppContext * a_app_context) 
 {
-        FILE *file_ptr = NULL;
-        int compressed = 0, c1 = 0, c2 = 0;
-        gzFile zfile_ptr = NULL;
-        int size = 1024,
-                num_of_chars_read = 0,
-                parse_status = MLVIEW_OK;
-        gchar file_buffer[size] ;
+        GnomeVFSResult vfs_result = GNOME_VFS_OK;
+        GnomeVFSHandle *vfs_handle;
+        GnomeVFSFileSize  bytes = 1024, read_bytes;
+        gchar buffer[bytes];
+        int parse_status = MLVIEW_OK;
         xmlParserCtxt *xml_ctxt = NULL;
         struct MlViewAppSettings *settings = NULL ;
 
@@ -234,47 +233,30 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
         xmlDoValidityCheckingDefaultValue = 0;
 
         if (a_app_context) {
-                settings = mlview_app_context_get_settings
-                        (a_app_context) ;
+                settings = mlview_app_context_get_settings (a_app_context) ;
                 g_return_val_if_fail (settings, MLVIEW_ERROR) ;
         }
 
         /*now, open the xml file... */
-        g_return_val_if_fail (a_xml_file_name != NULL,
+        g_return_val_if_fail (a_xml_file_uri != NULL,
                               MLVIEW_NULL_FILE_NAME_ERROR);
 
-        file_ptr = fopen (a_xml_file_name, "rb");
-        if (!file_ptr)
-                return MLVIEW_BAD_FILE_ERROR ;
 
-        c1 = fgetc (file_ptr);
-        c2 = fgetc (file_ptr);
-        fclose (file_ptr);
-        if ((c1 == 0x1F) && (c2 == 0x8B)) {
-                compressed = 1;
-        } else {
-                compressed = 0;
-        }
+        vfs_result = gnome_vfs_open (&vfs_handle, a_xml_file_uri, GNOME_VFS_OPEN_READ);
+        g_return_val_if_fail (vfs_result == GNOME_VFS_OK, MLVIEW_ERROR);
 
-        zfile_ptr = gzopen (a_xml_file_name, "rb");
-
-        g_return_val_if_fail (zfile_ptr != NULL,
-                              MLVIEW_BAD_FILE_ERROR);
-
-        num_of_chars_read = gzread (zfile_ptr, file_buffer, 4);
-        g_return_val_if_fail (num_of_chars_read > 0,
-                              MLVIEW_EMPTY_FILE_ERROR);
+        /* Read first four bytes to create the parser context */
+        vfs_result = gnome_vfs_read (vfs_handle, (gpointer)buffer, (GnomeVFSFileSize)4, &read_bytes);
+        g_return_val_if_fail (vfs_result == GNOME_VFS_OK, MLVIEW_ERROR);
+        
         /*
          *Force the parser to ignore ignorable white spaces so that
          *indentation can be made properly at save time.
          */
         xmlKeepBlanksDefault (0);
         gv_app_ctxt = a_app_context;
-        xml_ctxt =
-                xmlCreatePushParserCtxt (NULL, NULL,
-                                         file_buffer,
-                                         num_of_chars_read,
-                                         a_xml_file_name);
+        xml_ctxt = xmlCreatePushParserCtxt (NULL, NULL, buffer, (int)read_bytes, a_xml_file_uri);
+
         if (a_store_external_subset_info == TRUE) {
                 /*
                  *overload the externalSubset sax handler with our custom one
@@ -300,16 +282,20 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
         xmlSetGenericErrorFunc (a_app_context,
                                 (xmlGenericErrorFunc)
                                 mlview_app_context_bufferize_error);
-        while ((num_of_chars_read =
-                gzread (zfile_ptr, file_buffer, size)) > 0) {
-                parse_status =
-                        xmlParseChunk
-                        (xml_ctxt,
-                         file_buffer, num_of_chars_read, 0);
 
-                if (parse_status != XML_ERR_OK)
+        while (vfs_result == GNOME_VFS_OK) {
+                vfs_result = gnome_vfs_read (vfs_handle, buffer, bytes, &read_bytes);
+                if (vfs_result != GNOME_VFS_OK)
                         break;
+
+                if (read_bytes > 0) {
+                        parse_status = xmlParseChunk (xml_ctxt, (char*)buffer, (int) read_bytes, 0);
+
+                        if (parse_status != XML_ERR_OK)
+                                break;
+                }
         }
+        
         /*
          *call the xmlParseChunk once more 2 
          *consume all the data stored in 
@@ -317,8 +303,7 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
          */
         if (parse_status == XML_ERR_OK) {
                 parse_status =
-                        xmlParseChunk (xml_ctxt, file_buffer,
-                                       num_of_chars_read, 1);
+                        xmlParseChunk (xml_ctxt, buffer,(int)read_bytes, 1);
         }
         *a_parser_context = xml_ctxt;
         /*restore the error display stream of libxml */
@@ -332,15 +317,20 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
                 mlview_app_context_set_error_dialog_title
                         (a_app_context, NULL);
         }
-        gzclose (zfile_ptr);
+
+        /* close file */
+        vfs_result = gnome_vfs_close (vfs_handle);
+
         /*
          * If the file is compressed ... we set it inside the doc
-         */
+        
         if (compressed) {
                 printf ("Compression detected\n");
                 xmlSetDocCompressMode ((**a_parser_context).
                                        myDoc, 9);
         }
+        */
+        
         return (parse_status == XML_ERR_OK) ? 0 : parse_status;
 }
 
@@ -559,7 +549,6 @@ mlview_external_subset_sax_handler (void *a_ctxt,
         }
 }
 
-
 /**
  *This code has been copied and modified from libxml2.
  *GPL code is good for you.
@@ -702,6 +691,9 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
         gchar *public_id = NULL;
         gchar *system_id = NULL;
         xmlChar *resource = NULL;
+        gchar *base_uri = NULL;
+        gchar *tmp_str = NULL;
+        enum MlViewStatus status = MLVIEW_BAD_PARAM_ERROR;
 
         g_return_val_if_fail (a_app_context != NULL, TRUE);
         g_return_val_if_fail (a_ext_subs_def != NULL,
@@ -714,22 +706,48 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
 
         if (a_system_id != NULL) 
                 system_id = (gchar*)a_system_id ;
-        else system_id = (gchar*)"\"\"";
+        else 
+                system_id = (gchar*)"\"\"";
 
         *a_ext_subs_def = NULL; /*sanity initialization */
         system_id = g_strdup (system_id) ;
 
         mlview_utils_uri_is_relative (system_id, &is_relative) ;
-        if (is_relative == TRUE) {
-                gchar *tmp_str = NULL ;
-                mlview_utils_relative_uri_to_absolute_uri (system_id,
-                                                           a_doc->URL,
-                                                           &tmp_str) ;
-                if (tmp_str) {
-                        system_id = tmp_str ;
-                        tmp_str = NULL ;
+
+        if (is_relative == TRUE) { 
+                base_uri = mlview_utils_get_dir_name_from_uri (a_doc->URL);
+
+                if (!base_uri) {
+                        mlview_utils_trace_info ("Can't get base URI.");
+
+                        g_free (system_id);
+
+                        system_id = (gchar*)"\"\"";
+                } else {
+                        status = mlview_utils_relative_uri_to_absolute_uri (system_id,
+                                                                            base_uri,
+                                                                            &tmp_str);
+                        
+                        g_free (base_uri);
+                        base_uri = NULL;
+
+                        if (!tmp_str || status != MLVIEW_OK) {
+                                mlview_utils_trace_info ("Can't get absolute URI.");
+
+                                g_free (system_id);
+                                
+                                system_id = (gchar*)"\"\"";
+                        }
+                        else {
+                                g_free (system_id);
+
+                                system_id = tmp_str;
+                                
+                                tmp_str = NULL;
+                        }
                 }
         }
+
         resource =
                 mlview_resolve_external_entity (a_app_context,
                                                 a_external_id,
@@ -874,7 +892,6 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
         return result;
 }
 
-
 /**
  *The callback of the libxml2 hash table scan method xmlHashScan () ;
  *Takes a_hash_data 
@@ -1124,7 +1141,11 @@ g_list_compare_string_elems (gchar * a_str1, gchar * a_str2)
  ******************************************/
 
 /**
- *Loads an xml file located on the disk.
+ *Loads an xml file located on the disk. The user not prompted
+ *at all, all informations about any DTD is forgotten.
+ *If no DTD name is specified or if there was an error while 
+ *loading the specified DTD, the returned document's extSubset
+ *will be NULL.
  *@param a_file_name the name of the file to open and parse.
  *@param a_app_context the application context to use. 
  *If set to NULL, 
@@ -1132,16 +1153,86 @@ g_list_compare_string_elems (gchar * a_str1, gchar * a_str2)
  *@return the xmlDoc resulting from 
  *the parsing or NULL if the parsing failed. 
  */
-xmlDoc *
-mlview_parsing_utils_load_xml_file (gchar * a_file_name,
-                                    MlViewAppContext *a_app_context)
+xmlDocPtr
+mlview_parsing_utils_load_xml_file_with_dtd (gchar *a_file_name,
+                                             gchar *a_dtd_name,
+                                             MlViewAppContext *a_app_context)
+{
+        xmlDoc *result = NULL;
+        xmlParserCtxtPtr parser_context = NULL;
+        xmlDtdPtr dtd = NULL;
+        
+        g_return_val_if_fail (a_file_name != NULL, NULL);
+        
+        if (strcmp (a_file_name, "")) {
+                gint load_res;
+                
+                /*
+                 *fixme: remember to add support of 
+                 *error handling while loading the document.
+                 */
+                load_res =
+                        load_xml_document_from_local_file
+                        (a_file_name, &parser_context,
+                         FALSE, a_app_context);
+                
+                if (parser_context == NULL) {
+                        mlview_app_context_error
+                                (a_app_context,
+                                  _
+                                 ("could not load xml document %s"),
+                                 a_file_name);
+                        
+                        return NULL;
+                }
+                
+                if (!load_res) {
+                        parser_context->myDoc->name = (xmlChar *)
+                                g_strdup (a_file_name);
+                        result = parser_context->myDoc;
+                }
+        }
+        
+        if (result && a_dtd_name) {
+                dtd = mlview_parsing_utils_load_dtd (a_dtd_name,
+                                                     a_app_context);
+                
+                if (dtd) {
+                        result->standalone = TRUE;
+                        result->extSubset = dtd;
+                        
+                        dtd->doc = result;
+                }
+        }
+        
+        if (parser_context != NULL) {
+                xmlFreeParserCtxt (parser_context);
+        }
+        
+        return result;
+}
+ 
+/**
+ *Loads an xml file located on the disk. The user is prompted
+ *for informations if a DTD is specified in the document.
+ *Loads an xml file from an URI.
+ *@param a_file_uri the URI of the file to open and parse.
+ *@param a_app_context the application context to use. 
+ *If set to NULL, 
+ *it is not taken in account.
+ *@return the xmlDoc resulting from 
+ *the parsing or NULL if the parsing failed. 
+ */
+xmlDocPtr 
+mlview_parsing_utils_load_xml_file_with_dtd_interactive (gchar *a_file_uri,
+                                                         MlViewAppContext *a_app_context)
 {
         xmlDoc *result = NULL;
         xmlParserCtxtPtr parser_context = NULL;
 
-        g_return_val_if_fail (a_file_name != NULL, NULL);
+        g_return_val_if_fail (a_file_uri != NULL, NULL);
 
-        if (strcmp (a_file_name, "")) {
+        if (strcmp (a_file_uri, "")) {
                 gint load_res;
 
                 /*
@@ -1150,7 +1241,7 @@ mlview_parsing_utils_load_xml_file (gchar * a_file_name,
                  */
                 load_res =
                         load_xml_document_from_local_file
-                        (a_file_name, &parser_context,
+                        (a_file_uri, &parser_context,
                          TRUE, a_app_context);
 
                 if (parser_context == NULL) {
@@ -1158,14 +1249,14 @@ mlview_parsing_utils_load_xml_file (gchar * a_file_name,
                                 (a_app_context,
                                  _
                                  ("could not load xml document %s"),
-                                 a_file_name);
+                                 a_file_uri);
 
                         return NULL;
                 }
 
                 if (!load_res) {
                         parser_context->myDoc->name = (xmlChar *)
-                                g_strdup (a_file_name);
+                                g_strdup (a_file_uri);
                         result = parser_context->myDoc;
                 }
         }
@@ -1193,7 +1284,6 @@ mlview_parsing_utils_save_xml_doc (xmlDoc * a_xml_doc,
         return xmlSaveFormatFile (a_file_path, a_xml_doc, 1);
 }
 
-
 /**
  *Loads an external DTD subset entity.
  *@param a_subset_def the definition of the external DTD subset to load.
@@ -1337,9 +1427,7 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
 /**
  *found during the parsing of the last xml document if and only if 
  *the parsing has been triggered by calling the
- *function mlview_parsing_utils_load_xml_file(gchar * a_file_name).
- *Note that after a given xml document parsing, 
- *this function can only be 
+ *function mlview_parsing_utils_load_xml_file_interactive_doctype.
  *
  *@return NULL. This function can also return NULL if no external 
  *subset has been declared in the 
@@ -1373,11 +1461,12 @@ MlViewExtSubsDef *
 mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
                                             gchar * a_title) 
 {
-        MlViewFileSelection *filesel = NULL;
+        gchar *dtd = NULL;
         MlViewExtSubsDef *subset_def = NULL;
+#ifndef MLVIEW_WITH_GTK_FILE_CHOOSER
+        MlViewFileSelection *filesel = NULL;
 
         enum MLVIEW_SELECTED_BUTTON button_clicked;
-        gchar *dtd = NULL;
 
         g_return_val_if_fail (a_app_context != NULL, NULL);
 
@@ -1410,6 +1499,37 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
                 break;
         }
 
+#else
+	gint response = 0;
+	GtkWidget *file_dialog = GTK_WIDGET (mlview_app_context_get_file_chooser 
+                                             (a_app_context, a_title, MlViewFileChooserOpenMode)) ;
+
+        g_return_val_if_fail (file_dialog != NULL, NULL);
+
+        mlview_app_context_sbar_push_message
+                (a_app_context,
+		 _("Choose a dtd file"));
+
+	response = gtk_dialog_run (GTK_DIALOG(file_dialog));
+	gtk_window_set_modal (GTK_WINDOW (file_dialog), FALSE);
+	gtk_widget_hide (GTK_WIDGET (file_dialog));
+
+	if (response == GTK_RESPONSE_OK) {
+                dtd = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_dialog));
+		
+                if (dtd) {
+                        subset_def =
+                                mlview_ext_subs_def_new (NULL, NULL, 
+                                                         dtd);
+                }
+                if (dtd) {
+                        g_free (dtd);
+                        dtd = NULL;
+                }
+	}
+
+        mlview_app_context_sbar_pop_message (a_app_context);
+#endif
         return subset_def;
 }
 
@@ -1426,6 +1546,7 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
  *relatively to the @a_current_xml_node.
  *@param a_app_context the current mlview application context.
  */
+#ifndef MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION
 gint 
 mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_app_context,
                                                          enum NODE_INSERTION_SCHEME a_insertion_scheme,
@@ -1524,6 +1645,14 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
                 }
 
                 break;
+        case CHANGE_CUR_ELEMENT_NAME:
+                if (a_current_xml_node->prev && a_current_xml_node->next) {
+                        nb_of_names = xmlValidGetValidElements (a_current_xml_node->prev,
+                                                                a_current_xml_node->next,
+                                                                feasible_names,
+                                                                MAX_NUMBER_OF_ELEMENT_NAMES_IN_CHOICE_LIST) ;
+                }
+                break ;
         default:
                 break;
         }                       /*end switch */
@@ -1571,7 +1700,76 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
         xmlSetGenericErrorFunc (NULL, NULL) ;
         return nb_of_names;
 }
+#else /*MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION*/
+gint
+mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_app_context,
+                                                         enum NODE_INSERTION_SCHEME a_insertion_scheme,
+                                                         xmlNode * a_current_xml_node,
+                                                         GList ** a_feasible_names_ptr)
+{
+        
+        xmlCompletionListType  type  ;
+        xmlChar *element_names[50] = {0} ;
+        int ret = 0, len = 0 ;
+        GList *feasible_element_names = NULL ;
 
+        g_return_val_if_fail (a_app_context, -2) ;
+                
+        switch (a_insertion_scheme) {
+        case INSERT_BEFORE:
+                type = XML_PREV_SIBLING_LIST_COMPLETION_TYPE ;
+                break ;
+        case INSERT_AFTER:
+                type = XML_NEXT_SIBLING_LIST_COMPLETION_TYPE ;
+                break ;
+        case ADD_CHILD:
+                type =  XML_APPEND_CHILD_LIST_COMPLETION_TYPE ;
+                break ;
+        case CHANGE_CUR_ELEMENT_NAME:
+                type = XML_CURRENT_ELEMENT_LIST_COMPLETION_TYPE
+                break ;
+        case CHANGE_CUR_ELEMENT_NAME:
+                type = XML_CURRENT_ELEMENT_LIST_COMPLETION_TYPE ;
+                break ;
+        default :
+                ret = -1 ;
+                goto cleanup ;
+                break ;
+        }
+        if (!ret)
+                len = xmlValidGetElementNameCompletionList
+			(a_current_xml_node, NULL, type,
+			 (const xmlChar**)element_names,
+			 50) ;
+        else
+                goto cleanup ;
+
+        if (len) {
+                int i = 0 ;
+                for (i = 0 ; i < len ; i++) {
+                        feasible_element_names = g_list_prepend (feasible_element_names,
+                                                                 element_names[i]) ;
+                }
+                if (feasible_element_names) {
+                        g_list_reverse (feasible_element_names) ;
+                }
+        }
+        if (!ret) {                
+                ret = len ;
+                *a_feasible_names_ptr = g_list_concat (*a_feasible_names_ptr,
+                                                       feasible_element_names) ;
+                feasible_element_names = NULL ;
+        }
+
+ cleanup:
+        if (feasible_element_names) {
+                g_list_free (feasible_element_names) ;
+                feasible_element_names = NULL ;
+        }
+        return ret ;
+}
+
+#endif
 /**
  *Builds the atttribute completion list of a given xml document.
  *@param a_app_context the mlview application context.
@@ -2151,3 +2349,340 @@ mlview_parsing_utils_build_graphical_attr_values (MlViewAppContext * a_app_conte
         return result;
 }
 
+xmlDtdPtr
+mlview_parsing_utils_load_dtd (const gchar *a_url,
+                               MlViewAppContext *a_ctxt)
+{
+        MlViewExtSubsDef *sd = NULL;
+        xmlDtdPtr dtd = NULL;
+
+        g_return_val_if_fail (a_url, NULL);
+        g_return_val_if_fail (a_ctxt && 
+                              MLVIEW_IS_APP_CONTEXT (a_ctxt), 
+                              NULL);
+
+        sd = mlview_ext_subs_def_new (NULL, NULL, a_url);
+
+        if (!sd)
+                goto cleanup;
+
+        dtd = mlview_parsing_utils_load_a_dtd (sd, a_ctxt);
+
+        if (!dtd)
+                goto cleanup;
+
+        return dtd;
+
+ cleanup:
+        if (sd) {
+                mlview_ext_subs_def_destroy (sd);
+                sd = NULL;
+        }
+
+        if (dtd) {
+                xmlFreeDtd (dtd);
+                dtd = NULL;
+        }
+
+        return NULL;
+}
+
+/* About the use of mlview_parsing_utils_load_xml_file_with_dtd (x, NULL, y) in
+   functions which load schemas : I hope it works (I don't think there are DTD
+   specified in schemas :-) but user should not be prompted for a DTD when a 
+   schema is loaded, I think :-) */
+
+xmlRelaxNGPtr 
+mlview_parsing_utils_load_rng (const gchar *a_url,
+                               MlViewAppContext *a_ctxt)
+{
+        xmlDocPtr doc = NULL;
+        xmlRelaxNGPtr rng = NULL;
+        xmlRelaxNGParserCtxtPtr ctx = NULL;
+
+        g_return_val_if_fail (a_url, NULL);
+        g_return_val_if_fail (a_ctxt && 
+                              MLVIEW_IS_APP_CONTEXT (a_ctxt), 
+                              NULL);
+
+        doc = mlview_parsing_utils_load_xml_file_with_dtd ((gchar*)a_url, NULL, a_ctxt);
+
+        if (!doc)
+                goto cleanup;
+
+        ctx = xmlRelaxNGNewDocParserCtxt (doc);
+
+        if (!ctx)
+                goto cleanup;
+
+        rng = xmlRelaxNGParse (ctx);
+
+        if (!rng)
+                goto cleanup;
+
+        xmlFreeDoc (doc);
+        doc = NULL;
+
+        xmlRelaxNGFreeParserCtxt (ctx);
+        ctx = NULL;
+
+        return rng;
+
+ cleanup:        
+        if (doc) {
+                xmlFreeDoc (doc);
+                doc = NULL;
+        }
+
+        if (ctx) {
+                xmlRelaxNGFreeParserCtxt (ctx);
+                doc = NULL;
+        }
+
+        if (rng) {
+                xmlRelaxNGFree (rng);
+                rng = NULL;
+        }
+
+        return NULL;
+}
+
+xmlSchemaPtr
+mlview_parsing_utils_load_xsd (const gchar *a_url,
+                               MlViewAppContext *a_ctxt)
+{
+        xmlDocPtr doc = NULL;
+        xmlSchemaPtr xsd = NULL;
+        xmlSchemaParserCtxtPtr ctx = NULL;
+
+        g_return_val_if_fail (a_url, NULL);
+        g_return_val_if_fail (a_ctxt && 
+                              MLVIEW_IS_APP_CONTEXT (a_ctxt), 
+                              NULL);
+
+        doc = mlview_parsing_utils_load_xml_file_with_dtd ((gchar*)a_url, NULL, a_ctxt); 
+
+        if (!doc)
+                goto cleanup;
+
+        ctx = xmlSchemaNewDocParserCtxt (doc);
+
+        if (!ctx)
+                goto cleanup;
+
+        xsd = xmlSchemaParse (ctx);
+
+        if (!xsd)
+                goto cleanup;
+
+        xmlFreeDoc (doc);
+        doc = NULL;
+
+        xmlSchemaFreeParserCtxt (ctx);
+        ctx = NULL;
+
+        return xsd;
+
+ cleanup:        
+        if (doc) {
+                xmlFreeDoc (doc);
+                doc = NULL;
+        }
+
+        if (ctx) {
+                xmlSchemaFreeParserCtxt (ctx);
+                doc = NULL;
+        }
+
+        if (xsd) {
+                xmlSchemaFree (xsd);
+                xsd = NULL;
+        }
+
+        return NULL;
+}
+
+void
+mlview_parsing_utils_clean_dtd (xmlDtdPtr a_dtd)
+{
+        xmlNodePtr cur;
+
+        g_return_if_fail (a_dtd);
+
+        a_dtd->doc = NULL;
+        
+        cur = a_dtd->children;
+
+        while (cur) {
+                if (cur->doc) 
+                        cur->doc = NULL;
+
+                cur = cur->next;
+        }
+}
+
+enum MlViewStatus
+mlview_parsing_utils_serialize_node_to_buf (const xmlNode *a_node,
+                                            gchar **out_buf)
+{
+        enum MlViewStatus status = MLVIEW_OK ;
+        xmlBuffer *xml_buffer = NULL ;
+        xmlChar *content = NULL ;
+
+        g_return_val_if_fail (a_node, MLVIEW_BAD_PARAM_ERROR) ;
+
+        xml_buffer = xmlBufferCreate () ;
+        if (!xmlNodeDump (xml_buffer, a_node->doc, (xmlNode*)a_node, 0, 0)){
+                mlview_utils_trace_info ("No byte were writen during serialization") ;
+                status = MLVIEW_ERROR ;
+                goto cleanup ;
+        }
+        if (!xml_buffer->use) {
+                status = MLVIEW_ERROR ;
+                goto cleanup ;
+        }
+        content = g_strndup (xmlBufferContent (xml_buffer),
+                             xml_buffer->use) ;
+        if (!content) {
+                status = MLVIEW_OUT_OF_MEMORY_ERROR ;
+                goto cleanup ;
+        }
+        *out_buf = content ;
+        content = NULL ;
+        
+ cleanup:
+        if (xml_buffer) {
+                xmlBufferFree (xml_buffer) ;
+                xml_buffer = NULL ;
+        }
+        return status ;
+}
+
+enum MlViewStatus
+mlview_parsing_utils_do_comment_node (const xmlNode *a_node,
+                                      xmlNode **a_comment_node)
+{
+        enum MlViewStatus status = MLVIEW_OK ;
+        xmlNode *comment_node = NULL ;
+        gchar *buf = NULL ;
+        xmlChar *encoded_buf = NULL ;
+
+        g_return_val_if_fail (a_node 
+                              && a_node->doc 
+                              && a_comment_node,
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        status = mlview_parsing_utils_serialize_node_to_buf (a_node, &buf) ;
+
+        if (status != MLVIEW_OK || !buf)
+                return status ;
+        encoded_buf = xmlEncodeEntitiesReentrant (a_node->doc,
+                                                  buf) ;
+        g_free (buf) ;
+        buf = NULL ;
+        comment_node = xmlNewDocComment (a_node->doc, encoded_buf) ;
+        if (!comment_node) {
+                status = MLVIEW_OUT_OF_MEMORY_ERROR ;
+                goto cleanup ;
+        }
+
+        *a_comment_node = comment_node ;
+        comment_node = NULL ;
+
+ cleanup:
+        if (buf) {
+                xmlMemFree (buf) ;
+                buf = NULL ;
+        }
+        if (comment_node) {
+                xmlFreeNode (comment_node) ;
+                comment_node = NULL ;
+        }
+        return status ;
+}
+
+enum MlViewStatus
+mlview_parsing_utils_parse_fragment (xmlDoc *a_doc,
+                                     const xmlChar *a_buf,
+                                     xmlNode **a_result_node)
+{
+        xmlDoc *doc = NULL ;
+        xmlNode *node = NULL ;
+        enum MlViewStatus status = MLVIEW_OK ;
+
+        doc = a_doc ;
+        if (!doc)
+                doc = xmlNewDoc ("1.0") ;
+        if (!doc) {
+                mlview_utils_trace_info ("Got a NULL document") ;
+                return MLVIEW_ERROR ;
+        }
+        if (xmlParseBalancedChunkMemory (doc, NULL, NULL, 0, 
+                                         a_buf, &node)) {
+                status = MLVIEW_ERROR ;
+                goto cleanup ;
+        }
+        
+        if (status == MLVIEW_OK) {
+                *a_result_node = node ;
+        }
+
+ cleanup:
+
+        if (doc && doc != a_doc) {
+                xmlFreeDoc (doc) ;
+                doc = NULL ;
+        }
+
+        return status ;
+}
+
+enum MlViewStatus
+mlview_parsing_utils_uncomment_node (xmlDoc *a_doc,
+		                     const xmlNode *comment_node,
+		                     xmlNode **a_result_node)
+{
+	enum MlViewStatus status = MLVIEW_OK ;
+	xmlParserCtxt *parser_context = NULL ;
+	xmlNode *uncommented_node = NULL ;
+	gchar *content = NULL, *unescaped_content = NULL ;
+
+	g_return_val_if_fail (a_doc && comment_node && a_result_node,
+			      MLVIEW_BAD_PARAM_ERROR) ;
+	g_return_val_if_fail (comment_node->type == XML_COMMENT_NODE,
+			      MLVIEW_BAD_PARAM_ERROR) ;
+
+	content = xmlNodeGetContent ((xmlNode*)comment_node) ;
+	g_return_val_if_fail (content, MLVIEW_ERROR) ;
+
+	parser_context = xmlCreateMemoryParserCtxt (content, strlen (content)) ;
+	g_return_val_if_fail (parser_context, MLVIEW_ERROR) ;
+
+	unescaped_content = xmlStringDecodeEntities (parser_context,
+			                             content, 
+						     XML_SUBSTITUTE_REF,
+						     0, 0, 0) ;
+
+	g_return_val_if_fail (unescaped_content, MLVIEW_ERROR) ;
+
+	status = mlview_parsing_utils_parse_fragment (a_doc, unescaped_content,
+			                              &uncommented_node) ;
+	if (status != MLVIEW_OK) {
+		uncommented_node = xmlNewText (content) ;
+	}
+	if (content) {
+		g_free (content) ;
+		content = NULL ;
+	}
+	if (parser_context) {
+		xmlFreeParserCtxt (parser_context) ;
+		parser_context = NULL ;
+	}
+	if (unescaped_content) {
+		g_free (unescaped_content) ;
+		unescaped_content = NULL ;
+	}
+
+	*a_result_node = uncommented_node ;
+	return MLVIEW_OK ;
+}
