@@ -23,9 +23,11 @@
  *
  *See COPYRIGHT file for copyright information.
  */
+#define ENABLE_NLS 1
 
 #include "config.h"
 #include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <libxml/parser.h>
 #include "mlview-app-context.h"
 
@@ -70,6 +72,25 @@ G_BEGIN_DECLS
         MLVIEW_PATH_TOO_DEEP_ERRROR,
         MLVIEW_NO_DOC_IN_VIEW_ERROR,
         MLVIEW_INT_SUBSET_ALREADY_PRESENT_ERROR,
+        MLVIEW_KEY_INPUTS_FULL_ERROR,
+        MLVIEW_KEY_BINDING_NOT_FOUND_ERROR,
+        MLVIEW_NOT_A_KEY_BINDING_ERROR,
+        MLVIEW_KEY_SEQUENCE_TOO_SHORT_ERROR,
+        MLVIEW_STACK_TOO_SHORT_ERROR,
+        MLVIEW_ENTRY_EMPTY_ERROR,
+        MLVIEW_STACK_EMPTY_ERROR,
+        MLVIEW_CANT_UNDO_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE1_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE2_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE3_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE4_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE5_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE6_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE7_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE8_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE9_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE10_ERROR,
+        MLVIEW_RESERVED_FOR_FUTURE_USE11_ERROR,
         /*must be the last one */
         MLVIEW_ERROR
 };
@@ -269,6 +290,13 @@ struct NameValuePair * mlview_utils_name_value_pair_new (GString *a_name,
 void mlview_utils_name_value_pair_free (struct NameValuePair *a_this,
                                         gboolean a_free_strings) ;
 
+
+enum MlViewStatus mlview_utils_get_current_word_bounds (gchar *a_phrase,
+                                                        gint a_phrase_len,
+                                                        gint a_cur_index,
+                                                        gchar **a_word_start,
+                                                        gchar **a_word_end) ;
+
 void  mlview_utils_name_value_pair_list_free (GList *a_nv_pair_list,
                                               gboolean a_free_strings) ;
 
@@ -342,6 +370,15 @@ int xmlValidGetValidElementsChildren (xmlNode * a_node,
 int xmlSetEntityNodeName (xmlDtd *aDtd, xmlEntity *anEntityNode,
                           xmlChar *aName) ;
 
+int xmlNodeIsChildOf (xmlNodePtr aChild,
+                      xmlNodePtr aParent);
+
+int xmlGetValidElementsChildren2 (xmlNode *aNode,
+                                  const xmlChar **aList,
+                                  unsigned int aMax) ;
+
+xmlNode * mlview_utils_get_last_child_element_node (xmlNode *a_ref_node) ;
+
 gint gtk_clist_row_absolute_top_ypixel (GtkCList * a_clist,
                                         gint a_row);
 
@@ -363,6 +400,7 @@ enum MlViewStatus mlview_utils_gtk_tree_view_expand_row_to_depth (GtkTreeView *a
 enum MlViewStatus mlview_utils_gtk_tree_view_expand_row_to_depth2 (GtkTreeView *a_view,
                                                                    GtkTreeIter *a_iter,
                                                                    gint a_depth) ;
+
 gint gtk_clist_absolute_row_top_ypixel (GtkCList * a_clist, gint a_row);
 
 
@@ -370,6 +408,17 @@ void mlview_utils_init (void);
 
 void mlview_utils_cleanup (void);
 
+gchar *mlview_utils_normalize_text (const gchar *a_original,
+                                    const gchar *a_ex_endline,
+                                    const gchar *a_endline,
+                                    const gchar *a_space,
+                                    const guint a_line_length) ;
+
+gchar * mlview_utils_locate_file (const gchar *a_file_name) ;
+
+gchar *mlview_utils_get_unique_string (const gchar *a_prefix) ;
+
+gchar * mlview_utils_escape_underscore_for_gtk_widgets (const gchar *a_in_string) ;
 
 G_END_DECLS
 
