@@ -154,6 +154,11 @@ struct _MlViewXMLDocumentClass {
                                         xmlNs *a_ns,
                                         gpointer a_user_data) ;
 
+	void (*replace_node) (MlViewXMLDocument *a_this,
+			      xmlNode *old_node,
+			      xmlNode *new_node,
+			      gpointer a_user_data) ;
+
         void (*node_commented) (MlViewXMLDocument *a_this,
                                 xmlNode *a_node,
                                 xmlNode *a_new_node,
@@ -161,6 +166,7 @@ struct _MlViewXMLDocumentClass {
 
         void (*node_uncommented) (MlViewXMLDocument *a_this,
                                   xmlNode *a_node,
+				  xmlNode *a_new_node,
                                   gpointer a_user_data) ;
 
         void (*node_changed) (MlViewXMLDocument *a_xml_doc,
@@ -313,6 +319,11 @@ enum MlViewStatus mlview_xml_document_set_node_name (MlViewXMLDocument * a_xml_d
                                                      guchar * a_name,
                                                      gboolean a_emit_signal);
 
+enum MlViewStatus mlview_xml_document_set_node_name_without_xpath (MlViewXMLDocument *a_this,
+								   xmlNode *a_node,
+								   guchar *a_name,
+								   gboolean a_emit_signal) ;
+
 enum MlViewStatus mlview_xml_document_set_entity_node_name (MlViewXMLDocument *a_this,
                                                             xmlEntity *a_entity,
                                                             xmlDtd *a_dtd_node,
@@ -439,14 +450,17 @@ MlViewXMLDocument *mlview_xml_document_open_with_dtd (gchar * a_file_name,
 
 
 enum MlViewStatus mlview_xml_document_replace_node (MlViewXMLDocument *a_this,
-                                                    xmlNode *a_node,
+                                                    gchar *a_node_path,
                                                     xmlNode *a_replacement,
                                                     gboolean a_emit_signal) ;
 
 enum MlViewStatus mlview_xml_document_comment_node (MlViewXMLDocument *a_this,
-                                                    xmlNode *a_node,
+                                                    const gchar *a_node_path, 
                                                     gboolean a_emit_signal) ;
 
+enum MlViewStatus mlview_xml_document_uncomment_node (MlViewXMLDocument *a_this,
+						      const gchar *a_node,
+						      gboolean a_emit_signal) ;
 
 gchar *mlview_xml_document_get_uri (MlViewXMLDocument *a_this);
 
